from .client import Empty, HttpUrl, SDKClient, SDKResponse


class ClickAgainService:
    def __init__(self, client: SDKClient, url: HttpUrl):
        self._client = client
        self._url = url
        self.delete_url = self._url + "/api/delete_org_from_amocrm/?org_amo_id="

    def delete_amocrm_org(self, org_amo_id: int, token: str) -> SDKResponse[Empty]:
        return self._client.post(
            self.delete_url + f"{org_amo_id}",
            Empty,
            headers={
                "Content-Type": "application/json",
                "Authorization": f"TokenService {token}",
            },
            timeout=60,
        )



import json

from typing import List, Optional

from pydantic import BaseModel, HttpUrl

from sdk import Gender

from .client import Empty, SDKClient, SDKResponse


class Service(BaseModel):
    name: str
    price_code: str
    cost: str


class Client(BaseModel):
    uuid: str
    fio: str
    sex: str
    birth: str
    phone: Optional[str]
    email: Optional[str]
    nationality: Optional[str]
    city: Optional[str]
    okrug: Optional[str]
    street: Optional[str]
    building: Optional[str]
    google_cid: Optional[str]
    yandex_cid: Optional[str]


class LabOrderRequest(BaseModel):
    lab_number: str
    dc: str
    cost: str
    pre_record_number: Optional[str]
    mc_id: str
    client: Client
    services: List[Service]


class TopFive(BaseModel):
    price_code: str
    name: str


class PopServiceResponse(BaseModel):
    price_code: str
    top_five: List[TopFive]


class PriceCodeRequest(BaseModel):
    price_code: Optional[str]


class RecommendationRequest(BaseModel):
    price_code: List[str]
    sex: Optional[Gender]
    age: Optional[int]

    class Config:
        use_enum_values = True


class RecommendationService(BaseModel):
    price_code: str
    name: str


class RecommendationServiceResponse(BaseModel):
    recommendations: List[RecommendationService]


class RecommendationComplex(BaseModel):
    name: str
    services: List[RecommendationService]


class RecommendationComplexResponse(BaseModel):
    recommendations: List[RecommendationComplex]


class AnalyticsService:
    def __init__(self, client: SDKClient, url: HttpUrl, token: str):
        self._client = client
        self._url = url
        self._token = token

    def send_lab_order(self, query: LabOrderRequest, timeout=3) -> SDKResponse[Empty]:
        return self._client.post(
            self._url + "api/rfm/edit",
            Empty,
            data=json.dumps(query.dict()),
            timeout=timeout,
            headers={"Authorization": f"Bearer {self._token}"},
        )

    def top_services(
        self, query: PriceCodeRequest, timeout=3
    ) -> SDKResponse[List[PopServiceResponse]]:
        return self._client.get(
            self._url + f"api/recommend/top_five",
            PopServiceResponse,
            params=query.dict(),
            timeout=timeout,
            headers={"Authorization": f"Bearer {self._token}"},
        )

    def recommendations(
        self, query: RecommendationRequest, timeout=3
    ) -> SDKResponse[List[RecommendationServiceResponse]]:
        return self._client.get(
            self._url + f"api/recommend/recommendations",
            RecommendationServiceResponse,
            params=query.dict(exclude_none=True),
            timeout=timeout,
            headers={"Authorization": f"Bearer {self._token}"},
        )

    def recommendations_v2(
        self, query: RecommendationRequest, timeout=3
    ) -> SDKResponse[List[RecommendationComplexResponse]]:
        return self._client.get(
            self._url + f"api/recommend/v2/recommendations",
            RecommendationComplexResponse,
            params=query.dict(exclude_none=True),
            timeout=timeout,
            headers={"Authorization": f"Bearer {self._token}"},
        )



import hashlib
import hmac

from enum import Enum, IntEnum
from typing import Optional

from pydantic import BaseModel, Field, HttpUrl

from .client import Empty, SDKClient, SDKResponse


class ActionCode(IntEnum):
    """action_code=1 - Факт оформления заказа на сайте
       action_code=2 - Дошедший пациент записанный на анализы через сайт
       action_code=3 - Оплаченный заказ (медкнижа, справка в бассейн, профосмотр, флг, пцр ковид)"""

    PRE_RECORD = 1
    MC = 2
    PAY = 3


class ConfirmStatus(str, Enum):
    pending = "pending"
    approved = "approved"
    declined = "declined"


class AdmitAdRequest(BaseModel):
    uid: str = Field(description="admitad_code")
    order_id: str = Field(description="id заказа в мис")
    campaign_code: str = Field(description="одна из настроек интеграции")
    action_code: ActionCode = Field(description="Коды действий. Возможны 1/2/3.")
    tariff_code: int = Field(description="1 – Тариф по умолчанию", default=1)
    currency_code: str = Field(default="RUB")
    position_id: int = Field(description="Номер услуги в корзине")
    position_count: int = Field(description="Количество позиций в корзине")
    payment_type: str = Field(default="sale")
    product_id: str = Field(description="Код услуги")
    quantity: int = Field(description="Количество единиц услуги", default=1)
    postback: int = Field(default=1)
    postback_key: str = Field(description="Ключ авторизации")
    price: int = Field(description="Цена услуги")
    client_id: Optional[str]

    class Config:
        use_enum_values = True


class ConfirmRequest(BaseModel):
    campaign_code: str
    revision_key: str
    order_id: str
    status: ConfirmStatus
    comment: str = ""
    amount: float

    class Config:
        use_enum_values = True


class AdmitAdService:
    def __init__(self, client: SDKClient, url: HttpUrl = "https://ad.admitad.com/"):
        self._client = client
        self._url = url

    def send_position(self, query: AdmitAdRequest, timeout=3) -> SDKResponse[Empty]:
        return self._client.get(
            self._url + "r", Empty, params=query.dict(), timeout=timeout
        )

    def confirm_order(self, query: ConfirmRequest, timeout=3) -> SDKResponse[Empty]:
        campaign_code = query.campaign_code.encode("utf-8")
        order_id = query.order_id.encode("utf-8")
        revision_secret_key = query.revision_key.encode("utf-8")
        revision_sign = hmac.new(
            revision_secret_key, campaign_code + order_id, hashlib.sha1
        ).hexdigest()
        return self._client.get(
            self._url + "rp",
            Empty,
            params={
                "campaign_code": query.campaign_code,
                "revision_sign": revision_sign,
                "order_id": order_id,
                "status": query.status.value,
                "amount": query.amount,
                "comment": query.comment,
            },
            timeout=timeout,
        )



import datetime
import json

from typing import List, Optional

from pydantic import BaseModel, HttpUrl

from .client import Empty, SDKClient, SDKResponse


class Service(BaseModel):
    price_code: str
    cost: str


class ResultVisitRequest(BaseModel):
    number: int
    datetime_visit: datetime.datetime
    payment_cost: Optional[int]
    payment_date: Optional[datetime.datetime]
    client_phone: Optional[str]
    order_id: Optional[str]
    services: Optional[List[Service]]


class SiteService:
    def __init__(self, client: SDKClient, url: HttpUrl, token: str):
        self._client = client
        self._url = url
        self._token = token

    def send_result_visit(self, query: ResultVisitRequest, timeout=3) -> SDKResponse[Empty]:
        return self._client.get(
            self._url,
            Empty,
            data=json.dumps(query.dict()),
            timeout=timeout,
            headers={"Authorization": self._token},
        )



from typing import List, Optional
from urllib.parse import urljoin

from pydantic import BaseModel, HttpUrl, SecretStr

from .base import PdfResponse
from .client import Empty, SDKClient, SDKResponse
from .pre_record import PreRecordResponse


class TokenRequest(BaseModel):
    username: str
    password: SecretStr

    class Config:
        json_encoders = {
            SecretStr: lambda v: v.get_secret_value() if v else None,
        }


class RefreshTokenRequest(BaseModel):
    refresh: str


class TokenResponse(BaseModel):
    access: str
    refresh: Optional[str]


class RegisterRequest(BaseModel):
    email: str
    phone_number: str
    password: SecretStr

    class Config:
        json_encoders = {
            SecretStr: lambda v: v.get_secret_value() if v else None,
        }


class ActivateRequest(BaseModel):
    phone: str
    code: str


class ResendSmsRequest(BaseModel):
    phone: str


class GetOrderPdf(BaseModel):
    number: str


class UserResponse(BaseModel):
    last_name: Optional[str]
    first_name: Optional[str]
    middle_name: Optional[str]
    mis_uuid: Optional[str]
    username: str
    email: str
    phone_number: str
    birth: Optional[str]
    is_active: bool


class OrderResponse(BaseModel):
    lab_number: str
    pre_record_number: Optional[str]
    create_dt: str
    all_done_dt: Optional[str]
    pdf: Optional[str]


class OrdersResponse(BaseModel):
    results: List[OrderResponse]


class PreRecordsResponse(BaseModel):
    results: List[PreRecordResponse]


class PersonalService:
    def __init__(self, client: SDKClient, url: HttpUrl):
        assert url.endswith("/")
        self._client = client
        self._url = url

    def get_token(self, query: TokenRequest, timeout=3) -> SDKResponse[TokenResponse]:
        return self._client.post(
            urljoin(str(self._url), "v1/token/"),
            TokenResponse,
            data=query.json(),
            timeout=timeout,
            headers={"Content-Type": "application/json"},
        )

    def refresh_token(
        self, query: RefreshTokenRequest, timeout=3
    ) -> SDKResponse[TokenResponse]:
        return self._client.post(
            urljoin(str(self._url), "v1/token/refresh/"),
            TokenResponse,
            data=query.json(),
            timeout=timeout,
            headers={"Content-Type": "application/json"},
        )

    def register(self, query: RegisterRequest, timeout=3) -> SDKResponse[UserResponse]:
        return self._client.post(
            urljoin(str(self._url), "v1/lk/register/"),
            UserResponse,
            data=query.json(),
            timeout=timeout,
            headers={"Content-Type": "application/json"},
        )

    def me(self, token: str, timeout=3) -> SDKResponse[UserResponse]:
        return self._client.get(
            urljoin(str(self._url), "v1/lk/me/"),
            UserResponse,
            timeout=timeout,
            headers={
                "Content-Type": "application/json",
                "Authorization": f"Bearer {token}",
            },
        )

    def activate(self, query: ActivateRequest, timeout=3) -> SDKResponse[Empty]:
        return self._client.post(
            urljoin(str(self._url), "v1/lk/activate/"),
            Empty,
            data=query.json(),
            timeout=timeout,
            headers={"Content-Type": "application/json"},
        )

    def orders(self, token: str, timeout=10) -> SDKResponse[OrdersResponse]:
        return self._client.get(
            urljoin(str(self._url), "v1/lk/orders/"),
            OrdersResponse,
            timeout=timeout,
            headers={
                "Content-Type": "application/json",
                "Authorization": f"Bearer {token}",
            },
        )

    def pre_records(self, token: str, timeout=10) -> SDKResponse[PreRecordsResponse]:
        return self._client.get(
            urljoin(str(self._url), "v1/lk/pre_records/"),
            PreRecordsResponse,
            timeout=timeout,
            headers={
                "Content-Type": "application/json",
                "Authorization": f"Bearer {token}",
            },
        )

    def refresh_mis_data(self, token: str, timeout=10) -> SDKResponse[UserResponse]:
        return self._client.get(
            urljoin(str(self._url), "v1/lk/refresh/"),
            UserResponse,
            timeout=timeout,
            headers={
                "Content-Type": "application/json",
                "Authorization": f"Bearer {token}",
            },
        )

    def resend_sms_code(self, query: ResendSmsRequest, timeout=10) -> SDKResponse[Empty]:
        return self._client.post(
            urljoin(str(self._url), "v1/lk/resend_sms/"),
            Empty,
            data=query.json(),
            timeout=timeout,
            headers={
                "Content-Type": "application/json",
            },
        )

    def order_pdf(
        self, query: GetOrderPdf, token: str, timeout=10
    ) -> SDKResponse[PdfResponse]:
        return self._client.get(
            urljoin(str(self._url), f"v1/lk/{query.number}/pdf/"),
            PdfResponse,
            timeout=timeout,
            headers={
                "Content-Type": "application/pdf",
                "Authorization": f"Bearer {token}",
            },
        )



import datetime
import json

from enum import Enum
from typing import List, Optional
from urllib.parse import urljoin

from pydantic import BaseModel, Field, HttpUrl, root_validator

from . import Gender
from .client import Empty, SDKClient, SDKResponse


class SendTokenCodeRequest(BaseModel):
    phone: str


class AuthType(str, Enum):
    VK = "vk"
    SMS = "sms"


class TokenRequest(BaseModel):
    type: AuthType = Field(default=AuthType.SMS)
    phone: Optional[str]
    code: str

    class Config:
        use_enum_values = True

    @root_validator()
    def required_by_type(cls, values):
        auth_type = values.get("type")
        if auth_type == AuthType.SMS.value:
            raise ValueError(f"phone is required for {AuthType.SMS.value}")
        return values


class TokenResponse(BaseModel):
    token: str


class Status(BaseModel):
    label: str
    date: Optional[datetime.datetime]


class Result(BaseModel):
    created_at: datetime.datetime
    ready_at: Optional[datetime.datetime]
    result_link: Optional[HttpUrl]
    status: Status


class Subgroup(BaseModel):
    subgroup_name: str
    subgroup_id: int
    analysis: List[int]


class Analysis(BaseModel):
    analysis_id: int
    group_id: int
    analysis_type: str
    analysis_name: str
    price: int
    price_with_discount: int
    code: str
    period: str
    notice: Optional[str]
    tag: str
    description: str
    prepare: str
    cito_analysis: Optional[int]
    is_cito: bool
    bio_materials: List[str]
    med_centers: List[int]
    composition: List[int]
    results: List[Result]


class Group(BaseModel):
    group_id: int
    group_name: str
    group_type: str
    icon: Optional[HttpUrl]
    subgroups: List[int]
    analysis: List[Analysis]


class ResultsResponse(BaseModel):
    date: datetime.datetime
    number: str
    result_link: Optional[HttpUrl]
    groups: List[Group]


class ClientResponse(BaseModel):
    uuid: str
    mis_uuid: str
    last_name: str
    first_name: str
    middle_name: str
    email: Optional[str]
    phone_number: str
    birth: Optional[datetime.date]
    sex: Optional[Gender]


class ClientUpdateRequest(BaseModel):
    last_name: Optional[str]
    first_name: Optional[str]
    middle_name: Optional[str]
    email: Optional[str]
    birth: Optional[datetime.date]
    sex: Optional[Gender]


class MobileBackendService:
    def __init__(self, client: SDKClient, url: HttpUrl):
        assert url.endswith("/")
        self._client = client
        self._url = url

    def send_token_code(
        self, query: SendTokenCodeRequest, timeout=3
    ) -> SDKResponse[Empty]:
        return self._client.post(
            urljoin(str(self._url), "v2/auth/send_sms_code/?default_render"),
            Empty,
            data=query.json(),
            timeout=timeout,
            headers={"Content-Type": "application/json"},
        )

    def get_token(self, query: TokenRequest, timeout=3) -> SDKResponse[TokenResponse]:
        data = query.dict(exclude_none=True)
        if query.type == AuthType.SMS:
            data["code"] = int(data["code"])
        elif query.type == AuthType.VK:
            data["access_token"] = data["code"]
            del data["code"]
        return self._client.post(
            urljoin(str(self._url), "v2/auth/generate_token/?default_render"),
            TokenResponse,
            data=json.dumps(data),
            timeout=timeout,
            headers={"Content-Type": "application/json"},
        )

    def get_orders(self, token: str, timeout: int = 3) -> SDKResponse[ResultsResponse]:
        return self._client.get(
            urljoin(str(self._url), "v2/user/history/order/?default_render"),
            ResultsResponse,
            params={},
            timeout=timeout,
            headers={
                "Content-Type": "application/json",
                "Authorization": f"Token {token}",
            },
        )

    def me(self, token: str, timeout: int = 3) -> SDKResponse[ClientResponse]:
        params = {"mis_uuid_required": True, "default_render": True}
        return self._client.get(
            urljoin(str(self._url), "v2/user/"),
            ClientResponse,
            params=params,
            timeout=timeout,
            headers={
                "Content-Type": "application/json",
                "Authorization": f"Token {token}",
            },
        )

    def update_client(
        self, query: ClientUpdateRequest, token: str, timeout: int = 3
    ) -> SDKResponse[ClientResponse]:
        return self._client.post(
            urljoin(str(self._url), "v2/user/?default_render"),
            ClientResponse,
            data=query.json(exclude_none=True),
            timeout=timeout,
            headers={
                "Content-Type": "application/json",
                "Authorization": f"Token {token}",
            },
        )



from datetime import datetime
from typing import Dict, List, Optional

from pydantic import BaseModel, Field, Json

from .client import Empty, HttpUrl, SDKClient, SDKResponse


class AddressBookSource(BaseModel):
    addressBookSourceType: str  # Тип адресной книги, используется для фильтра (BIRTHDAY, PERIODIC, SIMPLE)
    byOptimalSendTime: Optional[bool]  # Персонализированное время отправки
    byTimeZone: Optional[bool]  # Учет местного времени контакта при отправке
    checkDuplicate: Optional[bool]  # Проверка дубликатов контактов в адресной книге
    contactCountId: Optional[int]  # ID количества контактов в адресной книге
    contactGroupIds: List[
        int
    ]  # Массив ID групп контактов в адресной книге. Не используется вместе с параметром segmentIds
    stopContactGroupIds: Optional[List[int]]  # Массив ID стоп-листов контактов
    period: Optional[str]  # Периодичность (ONCE, DAILY, WEEKLY, MONTHLY)
    segmentIds: Optional[
        List[int]
    ]  # Массив ID сегментов внутри групп контактов адресной книги, если группа контактов разделена на сегменты
    smoothSendMinutes: Optional[
        int
    ]  # Плавная отправка, задается в минутах (от 0 до 1440)


class TaskInfo(BaseModel):
    scenarioType: Optional[str]  # Тип сценария. Используется при "type": "EVENT_FLOW"


class EventSources(BaseModel):
    channel: Optional[
        str
    ]  # Канал триггера (EMAIL, FLASHCALL, HLR, PUSH, PUSH_WALLET, SMS, VIBER, VK, VOICE, WHATSAPP)
    eventType: str  # Тип события (CONTACT_MOBILE_EVENT, CONTACT_WEB_EVENT, STATE)
    eventValue: Optional[str]  # События, при которых совершается рассылка


class EventCollectorSettings(BaseModel):
    maxNewRegisterSec: Optional[
        int
    ]  # Время после совершения события в секундах, в течение которого рассылка не будет повторяться
    onlyUpdateEventSources: Optional[
        List[EventSources]
    ]  # Массив событий для совершения повторяющихся рассылок


class GeoSettings(BaseModel):
    antiFraudMins: Optional[int]  # Время в минутах для повторной отправки сообщения
    locationIds: Optional[List[int]]  # Массив ID геолокаций
    pushApplicationIds: Optional[
        List[int]
    ]  # Массив ID приложений, в которых будет проведена рассылка


class NoEventSourceSettings(BaseModel):
    # не указано в доке
    pass


class Template(BaseModel):
    pass


class SmsTemplate(Template, BaseModel):
    shortenUrl: Optional[bool]  # При true сокращает длину ссылок в тексте сообщения
    from_alias: str = Field(alias="from")  # Согласованное имя отправителя
    text: str  # Текст сообщения (до 17085 символов включительно при использовании кириллицы)
    validity: int  # Срок жизни сообщения в секундах (по умолчанию 0, от 0 до 3 суток)


class EmailTemplate(Template, BaseModel):
    sourceAddress: str  # Email-адрес отправителя
    sourceName: str  # Имя отправителя
    subject: str  # Тема письма
    htmlBody: str  # Тело письма в формате HTML
    plainText: Optional[
        str
    ]  # Тело письма в формате Plaintext, все теги отображаются как обычный текст
    attachmentIds: Optional[List[int]]  # Массив файлов, прикрепленных к письму


class VkTemplate(Template, BaseModel):
    routes: List[str]  # Список возможных каналов доставки через запятую ([VK, OK])
    deliveryPolicy: Optional[
        str
    ]  # По умолчанию any. Возможные значения: any, mobile_device_required, verified_phone_number
    validity: Optional[
        int
    ]  # Срок жизни сообщения в секундах. (от 60 до 86400 (по умолчанию))
    templateId: str  # ID шаблона сообщения в Devino
    templateData: Optional[Json]  # JSON-объект, где ключи - имена переменных в шаблоне
    callbackData: Optional[
        str
    ]  # Данные, которые будут указаны в коллбэке со статусом сообщения
    callbackUrl: Optional[
        str
    ]  # URL, на который система будет отправлять коллбэки при изменении статуса сообщения


class WhatsAppTemplate(Template, BaseModel):
    pass


class ViberTemplate(Template, BaseModel):
    from_alias: str = Field(
        alias="from"
    )  # Имя отправителя, которое используется при отправке сообщения.
    action: Optional[
        str
    ]  # URL или deep link, на который перейдет пользователь после нажатия на кнопку
    caption: Optional[str]  # Текст кнопки, не более 30 символов в кодировке UTF-
    image: Optional[str]  # URL изображения в форматах JPG, JPEG или PNG
    text: Optional[str]  # Текст сообщения в кодировке UTF-8
    validity: Optional[int]  # Срок жизни сообщения в секундах


class PushTemplate(Template, BaseModel):
    pass


class PushWalletTemplate(Template, BaseModel):
    pass


class Templates(BaseModel):
    smsTemplate: Optional[SmsTemplate]
    viberTemplate: Optional[ViberTemplate]
    emailTemplate: Optional[EmailTemplate]
    pushTemplate: Optional[PushTemplate]
    pushWalletTemplate: Optional[PushWalletTemplate]
    whatsAppTemplate: Optional[WhatsAppTemplate]
    vkTemplate: Optional[VkTemplate]


class Triggers(BaseModel):
    channel: str  # Канал триггера (EMAIL, FLASHCALL, HLR, PUSH, PUSH_WALLET, SMS, VIBER, VK, VOICE, WHATSAPP)
    type: str  # Тип триггера (ADDRESS_BOOK_SOURCE, TRAN_API_SOURCE, EVENT_SOURCE, NO_EVENT_SOURCE, EVENT_COLLECTOR, GEO_LOCATION)
    index: int  # ID триггера
    parentIndex: Optional[int]  # ID родительского триггера
    startTime: Optional[datetime]  # Дата старта триггера. Формат - YYYY-MM-DD hh:mm:ss
    endTime: Optional[datetime]  # Дата окончания триггера. Формат - YYYY-MM-DD hh:mm:ss
    eventSources: Optional[List[EventSources]]  # Массив событий для совершения рассылок
    eventCollectorSettings: Optional[
        EventCollectorSettings
    ]  # Параметры повторяющихся рассылок по событиям
    geoSettings: Optional[GeoSettings]  # Параметры гео-рассылок
    noEventSourceSettings: Optional[
        NoEventSourceSettings
    ]  # Содержит параметр: stateWaitMaxTimeSec (integer) - время ожидания события в секундах
    template: Optional[Templates]  # Массив шаблонов сообщений


class CreateOmniTaskRequest(BaseModel):
    addressBookSource: Optional[AddressBookSource]  # Параметры адресных книг
    channel: str  # Канал рассылки: EMAIL, OMNI, PUSH, SMS, VIBER, VK, WHATSAPP
    draftId: Optional[int]  # ID черновика рассылки
    taskInfo: Optional[TaskInfo]  # Тип сценария, который активирует триггерную рассылку
    taskName: str  # Название рассылки. От 1 до 100 символов
    triggers: List[Triggers]  # Параметры триггеров dict
    type: str  # Тип рассылки (SIMPLE - одноразовая рассылка)


class Fields(BaseModel):
    pass


class PushInfo(BaseModel):
    pass


class Contact(BaseModel):
    email: Optional[str]
    phone: Optional[str]
    pushInfo: Optional[List[PushInfo]]  # Данные push-уведомления.
    fields: Optional[List[Fields]]  # Параметры макросов (полей) контактов в списке


class Contacts(BaseModel):
    contact: Contact
    callbackUrl: Optional[str]
    callbackData: Optional[Dict]


class SendTransactionalMessageRequest(BaseModel):
    taskId: int  # ID сценария рассылки
    contactIds: Optional[List[str]]  # ID контактов из адресных книг
    contacts: Optional[List[Contacts]]
    templates: Optional[Dict[str, Template]]


class SendAnswers(BaseModel):
    code: str
    mergeKey: str
    result: str
    reasons: List[Dict]


class ErrorAnswers(BaseModel):
    code: str
    result: str
    description: str
    reasons: List[Dict]


class SendTransactionalMessageResponse(BaseModel):
    contactIds: List[str]  # Массив ID загруженных контактов
    sendAnswers: List[SendAnswers]  # Массив успешно отправленных сообщений
    errorAnswers: List[ErrorAnswers]  # Массив неуспешно отправленных сообщений


class DevinoService:
    def __init__(self, client: SDKClient, url: HttpUrl, token: str):
        self._client = client
        self._url = url
        self._token = token
        self.create_omni_scenario_url = self._url + "omni-task-api/tasks/"
        self.send_transactional_message_url = self._url + "omni-scheduler/api/messages/"

    def create_omni_scenario(
        self, query: CreateOmniTaskRequest, timeout=3
    ) -> SDKResponse[Empty]:
        """https://docs.devino.online/0.2.0/ru/http/omni/"""
        return self._client.post(
            self.create_omni_scenario_url,
            Empty,
            data=query.json(),
            headers={
                "Authorization": f"Key {self._token}",
                "Content-Type": "application/json",
            },
            timeout=timeout,
        )

    def send_transactional_message(
        self, query: SendTransactionalMessageRequest, timeout=3
    ) -> SDKResponse[SendTransactionalMessageResponse]:
        """https://docs.devino.online/0.2.0/ru/http/omni/#sending-transactional-messages"""
        return self._client.post(
            self.send_transactional_message_url,
            SendTransactionalMessageResponse,
            data=query.json(),
            headers={
                "Authorization": f"Key {self._token}",
                "Content-Type": "application/json",
            },
            timeout=timeout,
        )



from enum import Enum
from typing import List, Optional

from pydantic import BaseModel

from ..client import SDKClient, SDKResponse


class Rating(str, Enum):
    RED = "red"
    YELLOW = "yellow"
    GREEN = "green"
    NO_DATA = "no_data"


class GetOrgInfoRequest(BaseModel):
    key: str
    inn: str


class LegalName(BaseModel):
    short: str
    full: str
    readable: str
    date: str


class ShortFull(BaseModel):
    topoShortName: str
    topoValue: str

    def full(self) -> str:
        return f"{self.topoShortName} {self.topoValue}"


class ParsedAddress(BaseModel):
    regionName: Optional[ShortFull]
    district: Optional[ShortFull]
    city: Optional[ShortFull]
    settlement: Optional[ShortFull]
    street: Optional[ShortFull]
    house: Optional[ShortFull]
    bulk: Optional[ShortFull]
    flat: Optional[ShortFull]

    def full(self) -> str:
        address_info = []
        for field, value in self.__dict__.items():
            if value is not None:
                address_info.append(value.full())
        return ", ".join(address_info)


class LegalAddress(BaseModel):
    parsedAddressRF: ParsedAddress


class UL(BaseModel):
    kpp: str
    legalName: LegalName
    legalAddress: LegalAddress


class Summary(BaseModel):
    greenStatements: Optional[bool]
    yellowStatements: Optional[bool]
    redStatements: Optional[bool]


class BriefReport(BaseModel):
    summary: Summary

    @property
    def rating(self) -> Rating:
        # указываем худший рейтинг из возможных
        if self.summary.redStatements:
            return Rating.RED
        elif self.summary.yellowStatements:
            return Rating.YELLOW
        elif self.summary.greenStatements:
            return Rating.GREEN
        else:
            return Rating.NO_DATA


class GetOrgInfoResponse(BaseModel):
    inn: str
    ogrn: str
    UL: UL
    briefReport: BriefReport


class SubscribeOrgRequestParams(BaseModel):
    key: str
    append: str


class SubscribeOrgRequestData(BaseModel):
    body: str


class SubscribeOrgResponse(BaseModel):
    ogrn: str
    inn: str


class GetChangedOrgRequest(BaseModel):
    key: str
    date: str   # Дата в формате ГГГГ-ММ-ДД


class GetChangedOrgResponse(BaseModel):
    ogrn: str


class KonturApiService:
    def __init__(self, client: SDKClient):
        self._client = client
        self._url = "https://focus-api.kontur.ru"
        self._url_req = self._url + "/api3/req"
        self._url_monList = self._url + "/api3/monList"
        self._url_mon = self._url + "/api3/req/mon"

    def get_org_info(
        self, query: GetOrgInfoRequest, timeout=3
    ) -> SDKResponse[List[GetOrgInfoResponse]]:
        return self._client.get(
            self._url_req, GetOrgInfoResponse, params=query.dict(), timeout=timeout,
        )

    def subscribe_to_updates_by_org(
        self, query_params: SubscribeOrgRequestParams, query_data: SubscribeOrgRequestData, timeout=3
    ) -> SDKResponse[List[SubscribeOrgResponse]]:
        return self._client.post(
            self._url_monList, SubscribeOrgResponse, params=query_params.dict(), data=query_data.dict(), timeout=timeout,
        )

    def get_changed_org(
        self, query: GetChangedOrgRequest, timeout=3
    ) -> SDKResponse[List[GetChangedOrgResponse]]:
        return self._client.get(
            self._url_mon, GetChangedOrgResponse, params=query.dict(), timeout=timeout,
        )



from typing import List, Optional

from pydantic import Field, BaseModel, validator

from .client import SDKClient, SDKResponse


class NsiIdentifier(BaseModel):
    identifier: str


class NsiQuery(NsiIdentifier):
    version: str
    size: int = 200


class NsiFields(BaseModel):
    id: str
    name: str
    number: Optional[str] = None  # Factor code_29n
    subject: Optional[str] = None  # StateCode
    code_okato: Optional[str] = None  # StateCode
    p_id: Optional[str] = None  # Post


class NsiResponse(BaseModel):
    result: str
    resultText: str = None
    resultCode: str = None
    total: int = None
    data: List[NsiFields] = Field(alias="list")

    @validator("data", pre=True)
    def transform_data(cls, items):
        """Преобразуем eav формат в набор полей необходимых для справочника."""
        if not isinstance(items, list):
            return items
        transformed = []
        for row in items:
            required_keys = list(cls.__fields__["data"].type_.__fields__.keys())
            transformed_item = {}
            for column in row:
                name = column["column"].lower()
                if name in required_keys:
                    transformed_item[name] = column["value"]
            transformed.append(transformed_item)
        return transformed


class NsiVersionResponse(BaseModel):
    result: str
    resultText: str = None
    resultCode: str = None
    version: str
    rowsCount: int
    lastUpdate: str


class NsiService:
    def __init__(self, client: SDKClient, token: str) -> None:
        self._client = client
        self._url_data = "https://nsi.rosminzdrav.ru/port/rest/data/"
        self._url_passport = "https://nsi.rosminzdrav.ru/port/rest/passport"
        self._token = token

    def get_data(
        self, query: NsiQuery, timeout=3
    ) -> SDKResponse[NsiResponse]:
        return self._client.get(
            self._url_data,
            NsiResponse,
            params={
                "identifier": query.identifier,
                "size": query.size,
                "userKey": self._token,
                "version": query.version,
            },
            timeout=timeout,
        )

    def check_version(
        self, query: NsiIdentifier, timeout=3
    ) -> SDKResponse[NsiVersionResponse]:
        return self._client.get(
            self._url_passport,
            NsiVersionResponse,
            params={
                "identifier": query.identifier,
                "userKey": self._token,
            },
            timeout=timeout,
        )



from .client import Empty, HttpUrl, SDKClient, SDKResponse
from pydantic import BaseModel


class MakeACallRequest(BaseModel):
    phone: str
    operator: str


class PbxRtService:
    def __init__(self, client: SDKClient, url: HttpUrl):
        self._client = client
        self._url = url
        self.create_call_url = self._url + "mis_mm/call_back/create/"

    def create_call(self, query: MakeACallRequest, timeout=3) -> SDKResponse[Empty]:
        return self._client.post(
            self.create_call_url,
            Empty,
            data=query.json(),
            headers={
                "Content-Type": "application/json",
            },
            timeout=timeout,
        )
